# an additional function, mainly as an example of using conductance.py
# also note that this file can be executed (see the usage at bottom)

from conductance import *

def resistance(file, node1=False, node2=False, cdict={}, r_weights=False):
	"""Given the filename of a graph, returns the equivalent resistance
	between the node1 and node2, if specified, and otherwise returns all
	equivalent resistances and the nodes, as a tuple (ns,rs).
	
	This is a pretty useful function, but mainly a good example of using the
	library."""
	g = pydot.graph_from_dot_file(file)
	cdict = make_color_dictionary(g.get_edge_list(),cdict)
	ns = nodes(g)
	curs = currents(g,ns,cdict,r_weights)
	if node1 and node2:
		a = index(ns,node1)
		b = index(ns,node2)
		gab = conductance(curs,a,b)
		if gab==0: # infinite resistance
			return None
		else:
			return 1.0 / gab
	else:
		gs = conductances(curs)
		rs = []
		for gi in gs:
			ri = []
			for g in gi:
				if g==None: # infinite conductance / zero resistance
					r = 0
				elif g==0:
					r = None
				else:
					r = 1.0 / g
				ri.append(r)
			rs.append(ri)
		return ns,rs

if __name__=='__main__':
	if len(sys.argv)>1 and sys.argv[1]=='-r':
		off = 1
		r_flag = True
	else:
		off = 0
		r_flag = False
	if len(sys.argv)==4+off:
		r = resistance(sys.argv[1+off],sys.argv[2+off],sys.argv[3+off],r_weights=r_flag)
		print 'Resistance from %s to %s is %f' % (sys.argv[2+off],sys.argv[3+off], r)
	elif len(sys.argv)==2+off:
		print 'Resistances:'
		ns, rs = resistance(sys.argv[1+off],r_weights=r_flag)
		for n in ns:
			print '\t%8s' % n.name,
		print
		for i in range(len(ns)):
			print ns[i].name,
			for r in rs[i]:
				print '\t%f' % r,
			print
	else:
		print 'Usage: python resistance.py [-r] <filename> [<node name> <node name>]'
